#!/usr/bin/perl
#
# indexer.perl
#
# builds index of all artists, linked with index of songs,
#  sorted and grouped by artist then album.
# output from this is used by the dynamic catalog browser.
#
# imports:
#  settings:
#    TUNEBOT_ROOT    root of tunebot installation.
#    TUNEIX          path of tracks index file,
#                       lines in format /.../alb/art/trk.mp3
#    TUNETAGS        path of tags database file,
#                       lines in format /.../alb[/art[/trk]]:TAG=value
#    BANNER_STRING   identifying string for web page titles
#
#  files:
#    $TUNEBOT_ROOT/html/bodypart.html.head
#    $TUNEBOT_ROOT/html/bodypart.html.tail
#
# generates:
#  files:
#    $TUNEBOT_ROOT/html/artists.html          list of all artists
#    $TUNEBOT_ROOT/tune.autotags              indexing related tags
#

########################################################################
# perl script boilerplate.
# set tunebot root and load utility module.
#
BEGIN {
    $tunebot_root="XXX-the-tunebot-install-root";
    @INC=($tunebot_root."/lib", @INC);
}
use Tunebot;
#
# end of perl boilerplate
########################################################################

$cgibinurl=$configs{"CGI_BIN_URL"};

#
# index buckets are top-level divisions of the artist index.
#
@ixbktlist=("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
	   "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z");
## ixbkt
#
# get the index bucket tag for a string
#
sub ixbkt {
   my ($a) = @_;
   $a = substr($a, 0, 1);
   if ($a ge "A" && $a le "Z") {
	return $a;
   } elsif ($a ge "a" && $a le "z") {
	return uc($a);
   } else {
	return "A";
   }
}

#
# read the text tags database file into a hash indexed by {file, tag},
# generating the tied DB file as we go.
#
use DB_File;
#unlink "$tunebot_root/tune.tags.db", "$tunebot_root/tune.autotags.db";
tie %tags, "DB_File", "$tunebot_root/tune.tags.db", O_RDWR|O_CREAT|O_TRUNC, 0664;
Tunebot::suckInTagsFromFile;
tie %autotags, "DB_File", "$tunebot_root/tune.autotags.db", O_RDWR|O_CREAT|O_TRUNC, 0664;
$gid = getgrnam($configs{"TUNEGROUP"});
chown -1, $gid, "$tunebot_root/tune.tags.db", "$tunebot_root/tune.autotags.db";

## folded
#
# for sorting upper and lowercase together
#
sub folded { uc($a) cmp uc($b); }

#
# read master list of all tracks.
#  1. make sort key for each track and add to list (@sortks).
#      <index bucket, artist, year, album name, track file name>
#  2. make reverse hash from sortable artist to real artist name
#
@sortks = ();
open(TUNEIX, $configs{TUNEIX});
$i = 0;
while ($trackPath = <TUNEIX>) {
    chomp $trackPath;
    $canon = $trackPath;
    $trackPath =~ s/\.mp3$//;
    @raw = split('/', $trackPath);
    $n = $#raw;
    $artistPath = join("/", @raw[0..$n-2]);
    $artistName = $tags{$artistPath, "N"};
    if ($artistName eq "") {
	$artistName = $raw[$n-2];
    }
    $artistSort = $tags{$artistPath, "SORT"};
    if ($artistSort eq "") {
	$artistSort = $artistName;
	$artistSort =~ tr/a-z/A-Z/;
    }
    $firAlbIdByArtist{$artistName} = 0;
    $reverseArtist{$artistSort} = $artistName;
    if ($tags{$artistPath, "BOLD"} != 0) {
	$boldArtist{$artistSort} = 1;
    }
    $albumPath = join("/", @raw[0..$n-1]);
    $albumName = $tags{$albumPath, "N"};
    if ($albumName eq "") {
	$albumName = $raw[$n-1];
    }
    $albumDate = substr($tags{$albumPath, "DATE"}, 0, 4);
    if ($albumDate eq "") {
	$albumDate = "0000"
    }
    $trackName = $tags{$trackPath, "N"};
    if ($trackName eq "") {
	$trackName = $raw[$n];
    }
    # separators are TABs
    $sortk = ixbkt($artistSort)."	$artistSort	$albumDate	$albumName	".$raw[$n];
#    print "$canon\n";
#    print " SORTS ON $sortk\n";
#    print "  RENDERS AS $artistName/$albumName/$trackName\n";
#    print "   GOES INTO ".$coalbucket{ixbkt($artistSort)}."\n";
#    @sortks = (@sortks, $sortk);
    $sortks[$i++] = $sortk;
    $canons{$sortk} = $canon;
}
close(TUNEIX);

#
# we append indexing-related tags to the autotags file:
#    {track_path, IXA_TKN} = track_number
#
open(AUTOTAGS, ">" . $tunebot_root . "/tune.autotags");

#
# sort all tracks by sort key from above, and number them.
#
@sortks = sort folded @sortks;
$numAlbums=0;
$numArtists=0;
$numTracks=0;
$lastArtist="";
$lastAlbum="";
foreach $sortk (@sortks) {
    ++$numTracks;
    ($trackPath = $canons{$sortk}) =~ s/\.mp3$//;
    print AUTOTAGS $trackPath.":IXA_TKN=".$numTracks."\n";
    print AUTOTAGS "IXA/TK/".$numTracks.":PATH=$trackPath\n";
    $autotags{$trackPath, "IXA_TKN"}=$numTracks;
    $autotags{"IXA/TK/$numTracks", "PATH"}=$trackPath;
    @a=split(/	/,$sortk);
    $artistSort=$a[1];
    $albumSort=$a[3];
    ($albumPath = $trackPath) =~ s/\/[^\/]*$//;
    ($artistPath = $albumPath) =~ s/\/[^\/]*$//;
    if ($albumSort ne $lastAlbum || $artistSort ne $lastArtist) {
	++$numAlbums;
	print AUTOTAGS $albumPath.":IXA_ABN=".$numAlbums."\n";
	print AUTOTAGS $albumPath.":IXA_FTA=".$numTracks."\n";
	print AUTOTAGS "IXA/AB/".$numAlbums.":PATH=$albumPath\n";
	$autotags{$albumPath, "IXA_ABN"}=$numAlbums;
	$autotags{$albumPath, "IXA_FTA"}=$numTracks;
	$autotags{"IXA/AB/$numAlbums", "PATH"}=$albumPath;
	if ($artistSort ne $lastArtist) {
	    ++$numArtists;
	    $firAlbIdByArtist{$reverseArtist{$artistSort}} = $numAlbums;
	    if ($lastAlf ne ixbkt($artistSort)) {
		$lastAlf=ixbkt($artistSort);
		print AUTOTAGS "IXA/I/$lastAlf:IXA_ABN=$numAlbums\n";
		$autotags{"IXA/I/$lastAlf", "IXA_ABN"}=$numAlbums;
	    }
	}
	if (!$setArtistPath{$artistPath}) {
	    $setArtistPath{$artistPath} = 1;
	    print AUTOTAGS $artistPath.":IXA_FAA=".$firAlbIdByArtist{$reverseArtist{$artistSort}}."\n";
	    $autotags{$artistPath, "IXA_FAA"}=$firAlbIdByArtist{$reverseArtist{$artistSort}};
	}
    }
    $lastArtist=$artistSort;
    $lastAlbum=$albumSort;
}
print AUTOTAGS "IXA:NTK=$numTracks\n";
print AUTOTAGS "IXA:NAB=$numAlbums\n";
print AUTOTAGS "IXA:NAR=$numArtists\n";
$autotags{"IXA", "NTK"}=$numTracks;
$autotags{"IXA", "NAB"}=$numAlbums;
$autotags{"IXA", "NAR"}=$numArtists;

close(AUTOTAGS);

#
# grind out the overall artists index.
#
print " artists.html\n";
#
# html boilerplate
#
open(ARTISTS, ">" . $tunebot_root . "/html/artists.html");
print ARTISTS "<html>\n";
print ARTISTS "<head>\n";
print ARTISTS "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n";
print ARTISTS "<title>".$configs{BANNER_STRING}." Index of Artists</title>\n";
print ARTISTS "</head>\n";
open(I, $tunebot_root . "/html/bodypart.html.head");
while ($line = <I>) {
    print ARTISTS $line;
}
close(I);
print ARTISTS "<center><font size=+2>Index of Artists</font>\n";
print ARTISTS "<br>click to see albums</center>\n";

#
# artists a-z, shown in 3 columns
#
# make list of all sortable artist names (@sarts).
# make list of index buckets needed (%alfes).
#
@sarts = keys(reverseArtist);
foreach $k (@sarts) {
    $alfes{ixbkt($k)} = 1;
}
#
# add space to end of each artist name, to handle case where artist is
#  same as index bucket (X).
#
for (@sarts) { s/$/ / }
#
# sort artists and index buckets together for display
#
$narts = $#sarts + 1;
@sarts = sort folded (@sarts, keys(alfes));
#
# grind out table
#
print ARTISTS "<p><center><font size=-1><b> - ", $narts, " Artists - ", $numTracks, " Tracks - </b></font></center><p>\n";
#
# quick a-z links
#
$quicklinks="<p><center>".Tunebot::renderQuickLinks()."</center>\n";
print ARTISTS $quicklinks;
$narts = $#sarts + 1;
$nrows = int(($narts + 2) / 3);
print ARTISTS "<ul>\n";
print ARTISTS "<table width=90%>\n";
for ($i = 0; $i < $nrows; $i++) {
    print ARTISTS "<tr>\n";
    for ($j = 0; $j < $nrows * 3; $j += $nrows) {
	$k = $j + $i;
	if ($k < $narts) {
	    if ($alfes{$sarts[$k]} == 1) {
		print ARTISTS "<td width=33%><b><font size=+1>- </font>";
		print ARTISTS "<a href=\"$cgibinurl/dynindex.cgi?abn=".$autotags{"IXA/I/".$sarts[$k], "IXA_ABN"}."\">";
		print ARTISTS "<font size=+1 color=black><b>";
		print ARTISTS $sarts[$k];
		print ARTISTS "</b></font></a><font size=+1> -</font></b></td>";
	    } else {
		($sa = $sarts[$k]) =~ s/ $//;
		$a = $reverseArtist{$sa};
		$aa = $a;
		if ($boldArtist{$sa} != 0) {
		    $aa = "<font size=+1><b>" . $a . "</b></font>";
		}
		print ARTISTS "<td width=33%><a href=\"$cgibinurl/dynindex.cgi?abn=".$firAlbIdByArtist{$a}."\">", $aa, "</a></td>\n";
	    }
	}
    }
    print ARTISTS "</tr>\n";
}
print ARTISTS "</table>\n";
print ARTISTS "</ul>\n";

#
# another copy of the quick a-z links
#
print ARTISTS $quicklinks;

#
# html trailer boilerplate
#
open(I, $tunebot_root . "/html/bodypart.html.tail");
while ($line = <I>) {
    print ARTISTS $line;
}
close(I);
print ARTISTS "</html>\n";
close(ARTISTS);

