#
# Tunebot.pm
#
# 2001-6-2 manchek
#
# Tunebot perl module.
# Does all the tasty common stuff that tunebot programs need.
#

package Tunebot;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(%configs %tags %autotags %posts);
@EXPORT_OK = qw(%configs %tags %autotags %posts);

##################
#  startup code  #
#                #
#################

#
# read configuration file into %configs hash
# the config file is in (limiter) shell syntax, with lines of form:
#    VARIABLE=value.
# read in each line and stick into %configs, with the variable name
# as key.  expand quotes ["'] and backslashes.  expand $variable
# syntax using already-defined configs.
#

# start with TUNEBOT_ROOT defined, in case it's not set in file.
$configs{"TUNEBOT_ROOT"}=$main::tunebot_root;

sub parseSettings {
    while ($line = <I>) {
	if ($line =~ /^[A-Za-z0-9_]+=/) {
	    chomp $line;
	    ($var = $line) =~ s/=.*//;
	    ($val = $line) =~ s/^[^=]*=//;
	    $val2 = "";
	    $qm="";
	    while ($val ne "") {
		$c = substr($val, 0, 1);
		$val = substr($val, 1);
		if ($qm ne "") {
		    if ($qm eq "\$") {
			if ($qm eq $c) {
			    $val2 .= "\$";
			} else {
			    $val=$c . $val;
			    ($c = $val) =~ s/[^A-Za-z0-9_].*//;
			    $val =~ s/^[A-Za-z0-9_]*//;
			    $val2 .= $configs{$c};
			}
			$qm="";
		    } elsif ($qm eq "\\") {
			$val2 .= $c;
			$qm="";
		    } elsif ($qm eq $c) {
			$qm="";
		    } else {
			$val2 .= $c;
		    }
		} else {
		    if ($c eq "\"" || $c eq "'" || $c eq "\\" || $c eq "\$") {
			$qm=$c;
		    } else {
			$val2 .= $c;
		    }
		}
	    }
#	    print $var, " = \"", $val2 , "\"\n";
	    $configs{$var}=$val2;
	}
    }
}

open(I, $main::tunebot_root . "/settings");
parseSettings;
close(I);
open(I, $main::tunebot_root . "/controls");
parseSettings;
close(I);

$cgibinurl=$configs{"CGI_BIN_URL"};

###########################
#  url library functions  #
#                         #
##########################

## urldecode(url_encoded_string)
#
# convert a URL-encoded string to a regular string:
#  translate '+' to ' '.
#  expand characters encoded as 2 hex digits (%00).
#
sub urldecode {
    my ($a) = $_[0];
    my ($b) = "";
    $a =~ tr/+/ /;
    while ($a ne "") {
        my ($c) = substr($a, 0, 1);
        if ($c eq "%") {
            $b .= chr hex substr($a, 1, 2);
            $a =~ s/...//;
        } else {
            $b .= $c;
            $a =~ s/.//;
        }
    }
    return $b;
}

## urlencode(string)
#
# encode a regular string in URL syntax:
#  translate unsafe characters using '+' for space and %xx
#  notation for others.
#
sub urlencode {
    my ($a)=$_[0];
    $a =~ s/&/&amp;/g;
    $a =~ s/</&lt;/g;
    $a =~ s/>/&gt;/g;
    $a =~ s/"/&quot;/g;
    return $a;
}
#sub urlencode {
#    my ($a)=$_[0];
#    my ($b)="";
#    while ($a ne "") {
#        my ($c) = substr($a, 0, 1);
#	if ($c =~ /[a-zA-Z0-9]/) {
#            $b .= $c;
#	} else {
#            $b .= sprintf("%%%02x",ord($c));
#	}
#	$a =~ s/.//;
#    }
#    return $b;
#}

## suckInPost()
#
# parse stdin into a list of name-value pairs and stuff into hash %posts.
# this is used to support the POST http method with default encoding type
# application/x-www-form-urlencoded.
# stdin format is "name1=value1&name2=value2&...".
#
sub suckInPost {
    while ($line = <>) {
	chomp $line;
	@nvpairs = split('&', $line);
	for ($i = 0; $i <= $#nvpairs; $i++) {
	    @nv = split('=',@nvpairs[$i]);
	    if ($#nv == 1) {
		if ($debug > 0) {
		    print @nv[0] . "  =  " . @nv[1] . "\n";
		}
		$posts{@nv[0]} = urldecode(@nv[1]);
	    }
	}
    };
}

## suckInGet()
#
# parse environment QUERY_STRING into nv pairs and hash them.
# this supports the GET http method.
#
sub suckInGet {
    @nvpairs = split('&', $ENV{QUERY_STRING});
    for ($i = 0; $i <= $#nvpairs; $i++) {
	@nv = split('=',@nvpairs[$i]);
	if ($#nv == 1) {
	    if ($debug > 0) {
		print @nv[0] . "  =  " . @nv[1] . "\n";
	    }
	    $posts{@nv[0]} = urldecode(@nv[1]);
	}
    }
}

## suckInPostGet()
#
# suck in stdin and environment QUERY_STRING, to support either
# POST or GET http methods.
#
sub suckInPostGet {
    if ($debug > 0) {
	print "<p>\n";
	print "query string is: " . $ENV{QUERY_STRING} . "\n";
	print "<p>\n";
	print "i've got " . $#ARGV . " command args\n";
	for ($i = 0; $i <= $#ARGV; $i++) {
	    print "<br>" . $i . ". " . @ARGV[$i] . "\n";
	}
	print "<p>\n";
	print "<pre>\n";
    }
    suckInPost;
    suckInGet;
    if ($debug > 0) {
	print "</pre>\n";
    }
}

#############################
#  tags database functions  #
#                           #
############################

## suckInTags()
#
# make %tags hash available.
# key of hash is { file_path, tag_name }.
#
sub suckInTags {
    use DB_File;
    tie %tags, "DB_File", $main::tunebot_root."/tune.tags.db", O_RDONLY, 0644;
}

## suckInAutotags()
#
# make %autotags hash available.
# key of hash is { file_path, tag_name }.
#
sub suckInAutotags {
    use DB_File;
    tie %autotags, "DB_File", $main::tunebot_root."/tune.autotags.db", O_RDONLY, 0644;
}

## suckInTagsFromFile()
#
# read all track tags into %tags hash.
# key of hash is { file_path, tag_name }.
#
sub suckInTagsFromFile {
    open(TUNETAGS, $configs{TUNETAGS});
    while ($line = <TUNETAGS>) {
	chop $line;
	$i = index($line, ":");
	$j = index($line, "=", $i);
	$file = substr($line, 0, $i);
	$tag = substr($line, $i + 1, $j - $i - 1);
	$value = substr($line, $j + 1);
	if ($tag eq "SORT") {
	    $value =~ tr/a-z/A-Z/;
	}
	$tags{$file, $tag} = $value;
    }
    close(TUNETAGS);
}

## suckInAutotagsFromFile()
#
# read all track auto-generated tags into %autotags hash.
# key of hash is { file_path, tag_name }.
#
sub suckInAutotagsFromFile {
    open(AUTOTAGS, $main::tunebot_root."/tune.autotags");
    while ($line = <AUTOTAGS>) {
	chop $line;
	$i = index($line, ":");
	$j = index($line, "=", $i);
	$file = substr($line, 0, $i);
	$tag = substr($line, $i + 1, $j - $i - 1);
	$value = substr($line, $j + 1);
	$autotags{$file, $tag} = $value;
    }
    close(AUTOTAGS);
}

## lookupTagInFile(file,tag)
#
# look up a tag value directly in a file.
# not typically used with track databases, but useful for other files.
#
sub lookupTagInFile {
    my ($a) = @_[0];
    my ($i) = index($a, "\.mp3");
    if ($i >= 0) {
	$a = substr($a, 0, $i);
    }
    my ($t) = "";
    if (open(F, $a . ".t")) {
	while ($i = <F>) {
	    chop $i;
	    if (substr($i, 0, length(@_[1])) eq @_[1]) {
		$t = substr($i, length(@_[1]));
		break;
	    }
	}
	close(F);
    } elsif (open(F, $a)) {
	while ($i = <F>) {
	    chop $i;
	    if (substr($i, 0, length(@_[1])) eq @_[1]) {
		$t = substr($i, length(@_[1]));
		break;
	    }
	}
	close(F);
    }
    return $t;
}


##############################
#  html rendering functions  #
#                            #
#############################

## albumIndexUrl(track_path)
#
# take track path and return URL for album in index.
#
sub albumIndexUrl {
    my ($path) = $_[0];
    $path =~ s/\.mp3$//;
    my ($b) = $path;
    my ($i) = rindex($b, "/");
    if ($i >= 0) {
	$b = substr($b, 0, $i);
    }
    $i = $tags{$b, "N"};
    if ($i ne "") {
	my ($x) = $autotags{$b, "IXA_ABN"};
	if ($x eq "") {
	    return "";
	} else {
	    return "$cgibinurl/dynindex.cgi?abn=$x";
	}
    }
}       

## friendlyTrackTitle(track_path hot_track_name_link)
#
# take track path and render into nice format in html.
#
sub friendlyTrackTitle {
    my ($path) = $_[0];
    $path =~ s/\.mp3$//;
    my ($title) = $tags{$path, "N"};
    if ($title eq "") {
	$title = "<em>" . $path . "</em>";
    }
    if ($hotname != 0) {
	$title = "<a alt=PlayIt href=\"$cgibinurl/tuneput.cgi?" . $path . ".mp3\">" . $title . "</a>";
    }
    my ($art) = $tags{$path, "ARTIST"};
    if ($art ne "") {
	$title = $title . " <em>- " . $art . "</em>";
    }
    my ($b) = $path;
    my ($i) = rindex($b, "/");
    if ($i >= 0) {
	$b = substr($b, 0, $i);
    }
    my ($a) = $b;
    $i = rindex($a, "/");
    if ($i >= 0) {
	$a = substr($a, 0, $i);
    }
    $i = $tags{$a, "N"};
    if ($i ne "") {
#	    $title = $title . "<br><em>&nbsp;" . $i;
	my $x = $autotags{$a, "IXA_FAA"};
	if ($x eq "") {
	    $title = $title . "<br><em>&nbsp;" . $i;
	} else {
	    $title="$title<br><em>&nbsp;<a href=\"$cgibinurl/dynindex.cgi?abn=$x\">$i</a>";
	}
	$i = $tags{$b, "N"};
	if ($i ne "") {
#		$title = $title . " / " . $i;
	    my $x = $autotags{$b, "IXA_ABN"};
	    if ($x eq "") {
		$title = $title . " / " . $i;
	    } else {
		$title="$title / <a href=\"$cgibinurl/dynindex.cgi?abn=$x\">$i</a>";
	    }
	}
	$title = $title . "</em>";
    }
    return $title;
}

## friendlyAlbumTitle(album_path)
#
# take album path and render into nice html format.
#
sub friendlyAlbumTitle {
    my ($title) = "";
    my ($b) = @_;
    my ($a) = $b;
    my ($i) = rindex($a, "/");
    if ($i >= 0) {
	$a = substr($a, 0, $i);
    }
    my $x = $autotags{$a, "IXA_FAA"};
    my $y = $autotags{$b, "IXA_ABN"};
    $i = $tags{$b, "N"};
    if ($y eq "") {
	$title = $i;
    } else {
	$title="<a href=\"$cgibinurl/dynindex.cgi?abn=$y\">$i</a>";
    }
    $i = $tags{$a, "N"};
    if ($i ne "") {
	if ($x eq "") {
	    $title = $title . " - " . $i;
	} else {
	    $title="$title - <a href=\"$cgibinurl/dynindex.cgi?abn=$x\">$i</a>";
	}
    }
    return $title;
}

#
# index buckets are top-level divisions of the artist index.
#
@ixbktlist=("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
       "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z");
# renderQuickLinks
#
# generate a line of HTML for quick a-z index links
#
sub renderQuickLinks {
    my $quicklinks="<b>\n";
    my $i = "";
    my $c;
    foreach $c (@ixbktlist) {
	if ($autotags{"IXA/I/$c", "IXA_ABN"} ne "") {
	    $quicklinks=$quicklinks.$i."<a href=\"$cgibinurl/dynindex.cgi?abn=".$autotags{"IXA/I/$c", "IXA_ABN"}."\">".$c."</a>\n";
	} else {
	    $quicklinks=$quicklinks.$i.$c."\n";
	}
	$i = "-";
    }
    $quicklinks=$quicklinks."</b>\n";
    return $quicklinks;
}

1;

